package cn.cslg.pas.mapper;

import cn.cslg.pas.common.model.dto.ProductQueryPageDTO;
import cn.cslg.pas.common.model.vo.ProductGroupVO;
import cn.cslg.pas.common.model.vo.ProductVO;
import cn.cslg.pas.domain.Product;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.poi.sl.usermodel.Background;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 产品表的Mapper层接口
 *
 * @Author chenyu
 * @Date 2023/3/8
 */
@Repository
public interface ProductMapper extends BaseMapper<Product> {
    /**
     * 插入数据
     *
     * @param product 产品数据对象
     * @return 返回受影响的行数
     */
    int insert(Product product);

    /**
     * 根据id删除产品
     *
     * @param id 产品id
     * @return 返回受影响的行数
     */
    int deleteById(Integer id);

    /**
     * 根据产品id修改数据
     *
     * @param product 产品数据对象
     * @return 返回受影响的行数
     */
    int updateById(Product product);

    /**
     * 根据产品名称统计数量
     *
     * @param productName       产品名称
     * @param productCategoryId 产品类别id
     * @return 返回统计到的数量
     */
    int countByProductName(String productName, Integer productCategoryId);

    /**
     * 根据id统计数量
     *
     * @param id 产品id
     * @return 返回统计到的数量
     */
    int countById(Integer id);

    /**
     * 根据id查询产品名称
     *
     * @param id 产品id
     * @return 返回查询到的名字
     */
    String queryNameById(Integer id);

    /**
     * 根据所属产品类别id统计数量
     *
     * @param productCategoryId 所属产品类别id
     * @return 返回统计到的数量
     */
    int countByProductCategoryId(Integer productCategoryId);

    /**
     * 根据id查询数据
     *
     * @param id 产品id
     * @return 返回查询到的数据
     */
    ProductVO getStandardById(Integer id);

    /**
     * 分页查询数据
     *
     * @param productQueryPageDTO 分页信息
     * @return 返回查询到的数据
     */
    List<ProductVO> query(ProductQueryPageDTO productQueryPageDTO);

    /**
     * 根据产品类别分组分页查询数据
     *
     * @param productQueryPageDTO 分组分页信息
     * @return 返回查询到的数据
     */
    List<ProductGroupVO> queryByGroupProductCategoryId(ProductQueryPageDTO productQueryPageDTO);

    /**
     * 根据公司分组分页查询数据
     *
     * @param productQueryPageDTO 分组分页信息
     * @return 返回查询到的数据
     */
    List<ProductGroupVO> queryByGroupCompanyName(ProductQueryPageDTO productQueryPageDTO);


}
